from datetime import datetime
from urllib import parse

from pandas import DataFrame


class ReportGenerator:
    def get_header(self, ticker, df: DataFrame):
        session_dt = datetime.strptime(df["Datetime"], "%Y-%m-%d %H:%M:%S.%f").date()
        return f"""## {ticker}

**Date:** {session_dt}"""

    def get_footer(self, ticker, ticker_exchange):
        ticker_exchange_symbol = parse.quote_plus(
            "{}:{}".format(ticker_exchange, ticker)
        )
        return f"""[Trading View](https://www.tradingview.com/chart/?symbol={ticker_exchange_symbol}) | 
[Yahoo Finance](https://finance.yahoo.com/chart/{ticker}) |
[Robinhood](https://robinhood.com/stocks/{ticker})"""

    def prepare_output(self, ticker, df: DataFrame, body):
        ticker_exchange = df["Exchange"]
        md_post = f"""
{self.get_header(ticker, df)}

{body}

{self.get_footer(ticker, ticker_exchange)}

---"""
        return md_post


report_generator = ReportGenerator()



from pathlib import Path

import pandas as pd
from pandas import DataFrame


class TickerData:
    def __init__(self):
        self.exchanges = [
            ("nasdaq", "nasdaq-listed.csv"),
            ("amex", "amex-listed.csv"),
            ("nyse", "nyse-listed.csv"),
        ]

    def load_exchange_tickers(self) -> DataFrame:
        exchange_securities = [s for s in self._extract_data(self.exchanges)]
        return pd.concat(exchange_securities)

    def _extract_data(self, exchanges):
        for exchange, data_file in exchanges:
            df = self._load_from("data/{}".format(data_file))
            df["exchange"] = exchange
            yield df

    def _load_from(self, listing_path):
        securities_file = Path.cwd().joinpath(listing_path)
        return pd.read_csv(securities_file, index_col=["Symbol"])


ticker_data = TickerData()



import configparser
import os

from krider.utils import str_to_bool

LIVE_MODE = str_to_bool(os.getenv("LIVE"))


def config(key, default_value=None):
    c = configparser.ConfigParser()
    c.read("env.cfg")

    if LIVE_MODE:
        return c.get("LIVE", key) or default_value
    else:
        return c.get("DEV", key) or default_value



import logging


def init_logger():
    handlers = [
        logging.StreamHandler(),
    ]

    logging.basicConfig(
        handlers=handlers,
        format="%(asctime)s - %(filename)s:%(lineno)d - %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
        level=logging.INFO,
    )
    logging.captureWarnings(capture=True)



import click

from krider.stock_store import stock_store
from krider.tasks.historical_data_downloader import historical_data_downloader
from krider.tasks.volume_analysis import volume_analysis_task
from krider.utils.log_helper import init_logger

init_logger()


@click.group()
def cli():
    pass


@cli.command()
@click.option(
    "--period",
    help="Use periods to download data. Valid periods: 1d,5d,1mo,3mo,6mo,1y,2y,5y,10y,ytd,max",
    default="1mo",
    required=False,
)
@click.option(
    "--interval",
    help="Data interval. 1m,2m,5m,15m,30m,60m,90m,1h,1d,5d,1wk,1mo,3mo",
    required=True,
    default="60m",
)
@click.option(
    "--stocks",
    help="Download historical data and fill gaps for provided list of stocks. Eg. MSFT,TSLA,AAPL",
)
@click.option(
    "--datadir",
    help="Directory to store database file",
    default="."
)
def populate_data(interval, period, stocks, datadir):
    stock_store.init_database(datadir)
    result = historical_data_downloader.run_with(interval, period, stocks)
    click.echo(result, nl=False)


@cli.command()
@click.option(
    "--period",
    help="Number of entries to use when running volume analysis",
    required=True,
)
@click.option(
    "--stocks", help="Run analysis on provided list of stocks. Eg. MSFT,TSLA,AAPL",
)
@click.option(
    "--datadir",
    help="Directory to load database file",
    default="."
)
def volume_analysis(period, stocks, datadir):
    stock_store.init_database(datadir)
    result = volume_analysis_task.run_with(period, stocks)
    click.echo(result, nl=False)



import logging

import numpy as np
import pandas as pd
from pandas import DataFrame
from tqdm import tqdm

from krider.bot_config import config, LIVE_MODE
from krider.notifications.console_notifier import console_notifier
from krider.notifications.reddit_notifier import reddit_notifier
from krider.stock_store import stock_store
from krider.ticker_data import ticker_data
from krider.utils.report_generator import report_generator
from krider.utils.timing_decorator import timeit

pd.set_option("display.max_rows", None)
pd.set_option("display.max_columns", None)
pd.set_option("display.width", None)


class VolumeAnalysisTask:
    @timeit
    def run_with(self, period, stocks=None):
        exchange_tickers: DataFrame = ticker_data.load_exchange_tickers()

        if stocks:
            selected_stocks = stocks.split(",")
            exchange_tickers = exchange_tickers[
                exchange_tickers.index.isin(selected_stocks)
            ]

        collective_post = []

        for ticker, ticker_df in tqdm(exchange_tickers.iterrows()):
            logging.debug("Running analysis on {}".format(ticker))
            selected_data: DataFrame = stock_store.data_for_ticker(ticker, period)
            if selected_data.empty:
                continue

            if self._if_anomaly_found(selected_data):
                indicator_df = selected_data.iloc[0]
                body = self.output_body(indicator_df)
                report = report_generator.prepare_output(ticker, indicator_df, body)
                collective_post.append(report)

        logging.info(
            "Total {} stocks found with unusually high volume".format(
                len(collective_post)
            )
        )

        content = dict(
            title="[Daily] High Volume Indicator",
            flair_id=config("HIGH_VOLUME_FLAIR"),
            body="\n".join(collective_post),
        )
        if LIVE_MODE:
            reddit_notifier.send_notification(content)
        else:
            console_notifier.send_notification(content)
        return "All done"

    def output_body(self, df: DataFrame):
        session_volume = float(df["Volume"])
        mean_volume = float("{:.0f}".format(df["MeanVolume"]))
        md_post = f"""**Volume:** {session_volume:,.0f}

**Mean Volume:** {mean_volume:,.0f}"""
        return md_post

    def _back_test_anomalies(self, df):
        mean = np.mean(df["Volume"])
        df["Volume_Activity"] = df["Volume"] > (10 * mean)
        logging.debug(df.where(df["Volume_Activity"] == True).dropna())

    def _if_anomaly_found(self, df):
        mean = np.mean(df["Volume"])
        df["MeanVolume"] = mean
        previous_session_vol = df["Volume"].iloc[0]
        return previous_session_vol > (20 * mean)


volume_analysis_task = VolumeAnalysisTask()



import logging

import yfinance


class StockDataProvider:
    def download_between_dates(self, ticker, interval, start, end):
        logging.debug("Requesting ticker {}".format(ticker))
        opts = dict(
            tickers=ticker, interval=interval, start=start, end=end, progress=False
        )
        return yfinance.download(**opts)

    def download_for_period(self, ticker, period, interval):
        logging.debug("Requesting ticker {}".format(ticker))
        opts = dict(tickers=ticker, interval=interval, period=period, progress=False)
        return yfinance.download(**opts)

    def check(self, ticker):
        return self.download_for_period(ticker, "1d", "1d")


stock_data_provider = StockDataProvider()



import logging
from datetime import timedelta
from typing import Any

import pandas as pd
from pandas import DataFrame
from sqlalchemy import create_engine
from sqlalchemy.exc import IntegrityError, OperationalError

skip_days = timedelta(days=2)


class StockStore:
    db_connection: Any = None

    def init_database(self, datadir):
        db_engine = create_engine("sqlite:///{}/stockstore.db".format(datadir), echo=False)
        self.db_connection = db_engine.connect()

    def save(self, ticker, data: DataFrame):
        if not self.db_connection:
            self.init_database(".")

        self._create_table_if_required(ticker)
        try:
            data.to_sql(
                ticker,
                self.db_connection,
                if_exists="append",
                index=True,
                index_label="Datetime",
            )
        except IntegrityError as e:
            logging.warning("Unable to save data: {}".format(e.args[0]))

        logging.debug("Saving ticker: {} with data: {}".format(ticker, data.shape))

    def data_for_ticker(self, ticker, period):
        if not self.db_connection:
            self.init_database(".")

        sql = f"""
        select * from \"{ticker}\" order by Datetime desc limit {period};
        """
        try:
            pd_sql = pd.read_sql(sql, self.db_connection)
            return pd.DataFrame(pd_sql)
        except OperationalError as e:
            logging.debug(
                "Error when reading data for {} - {}".format(ticker, e.args[0])
            )
            return pd.DataFrame()

    def _create_table_if_required(self, ticker):
        self.db_connection.execute(
            f"""
        CREATE TABLE IF NOT EXISTS "{ticker}" (
            "Datetime" TIMESTAMP PRIMARY KEY, 
            "Open" FLOAT, 
            "High" FLOAT, 
            "Low" FLOAT, 
            "Close" FLOAT, 
            "Adj Close" FLOAT, 
            "Volume" BIGINT,
            "Exchange" TEXT
        )
        """
        )


stock_store = StockStore()



import logging
from datetime import datetime

import praw

from krider.bot_config import config


class RedditNotifier:
    def __init__(self):
        self.reddit = praw.Reddit(
            client_id=config("REDDIT_CLIENT_ID"),
            client_secret=config("REDDIT_CLIENT_SECRET"),
            username=config("REDDIT_USER"),
            password=config("REDDIT_PASSWORD"),
            user_agent=config("REDDIT_USERAGENT"),
        )
        self.sub_reddit_name = config("SUB_REDDIT")
        self.sub_reddit = self.reddit.subreddit(self.sub_reddit_name)

    def send_notification(self, content):
        logging.info("Posting to reddit: {}".format(self.sub_reddit_name))
        self.sub_reddit.submit(
            title="{} : {}".format(
                content.get("title"), datetime.now().strftime("%Y-%m-%d")
            ),
            selftext=content.get("body"),
            flair_id=content.get("flair_id"),
        )


reddit_notifier = RedditNotifier()



from functools import wraps

from time import time


def timeit(f):
    @wraps(f)
    def wrap(*args, **kwargs):
        ts = time()
        result = f(*args, **kwargs)
        print("{} took {}".format(f.__name__, time() - ts))
        return result

    return wrap



import os
from datetime import datetime

import sys
from setuptools import find_packages, setup

dependencies = ['click', 'yfinance', 'pandas', 'numpy', 'tqdm', 'sqlalchemy', 'praw']

# 'setup.py publish-test' shortcut
if sys.argv[-1] == 'publish-test':
    os.system('rm -r dist/*')
    os.system('python setup.py sdist')
    os.system('twine upload -r pypitest dist/*')
    sys.exit()
# 'setup.py publish' shortcut
if sys.argv[-1] == 'publish':
    os.system('rm -r dist/*')
    os.system('python setup.py sdist')
    os.system('twine upload dist/*')
    sys.exit()

version = datetime.now().strftime("%Y%m%d")

# Get the long description from the relevant file
try:
    # in addition to pip install pypandoc, might have to: apt install -y pandoc
    import pypandoc

    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError, OSError) as e:
    print("Error converting README.md to rst:", str(e))
    long_description = open('README.md').read()

setup(
    name='stock-rider',
    version='0.5.' + version,
    url='https://github.com/namuan/stock-rider',
    license='MIT',
    author='DeskRiders Dev',
    author_email='me@deskriders.dev',
    description='Stock Rider',
    long_description=long_description,
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    setup_requires=['wheel'],
    install_requires=dependencies,
    entry_points={
        'console_scripts': [
            'stock-rider = krider.main:cli',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Unix',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



import logging

from pandas import DataFrame
from tqdm import tqdm

from krider.stock_data_provider import stock_data_provider
from krider.stock_store import stock_store
from krider.ticker_data import ticker_data
from krider.utils.timing_decorator import timeit


class HistoricalDataDownloader:
    def _download_and_save_ticker_data(self, ticker, ticker_exchange, interval, period):
        logging.debug("Scanning {} for the last {}".format(ticker, period))
        try:
            data = stock_data_provider.download_for_period(ticker, period, interval,)
            if not data.empty:
                data["Exchange"] = ticker_exchange
                stock_store.save(ticker, data)
        except Exception as e:
            logging.warning(
                "Something went wrong when processing ticker {}. Continuing ...".format(
                    ticker
                ),
                e,
            )

    @timeit
    def run_with(self, interval, period, stocks=None):
        exchange_tickers: DataFrame = ticker_data.load_exchange_tickers()

        if stocks:
            selected_stocks = stocks.split(",")
            exchange_tickers = exchange_tickers[
                exchange_tickers.index.isin(selected_stocks)
            ]

        for ticker, ticker_df in tqdm(exchange_tickers.iterrows()):
            ticker_exchange = ticker_df["exchange"]
            self._download_and_save_ticker_data(
                ticker, ticker_exchange, interval, period
            )

        return "All done."


historical_data_downloader = HistoricalDataDownloader()
